# -*- coding: utf-8 -*-

# Standard imports.
import re

# Trac imports.
from trac.core import *
from trac.mimeview.api import Context

# Trac interfaces.
from trac.web.main import IRequestHandler

# Local imports.
from tracdiscussion.api import *

class LatestTopicRequestHandler(Component):
    """
        Handle requests for the latest topics in a discussion forum.
    """
    implements(IRequestHandler)
    # IRequestHandler methods.

    def match_request(self, req):
        if req.path_info == '/discussion/redirect':
            # Proces redirection request.
            req.redirect(req.args.get('redirect_url'))
        else:
            # Prepare regular requests.
            match = re.match(r'''/discussion/forum/(\d+)/latest-topic''',
              req.path_info)
            if match:
                resource_id = match.group(1)
                req.args['forum'] = resource_id
            return match

    def process_request(self, req):        
        # Get database access.
        db = self.env.get_db_cnx()
        cursor = db.cursor()
        
        # Create request context.
        context = Context.from_request(req)
        context.realm = 'discussion-latest-topic'
        context.cursor = cursor
        
        forum_id = req.args['forum']
        topic_id = self._get_latest_topic_id(context, forum_id)

        req.redirect(req.href.discussion('topic', topic_id))

    # Internal methods
    def _get_latest_topic_id(self, context, forum_id):
        values = {'forum_id' : forum_id }
        sql = ("SELECT id, forum, MAX(time) "
               "FROM topic "
               "WHERE forum = %(forum_id)s" % (values))
        context.cursor.execute(sql)
        for row in context.cursor:
            return row[0]
        return None